"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getClashNodeNames = exports.getClashNodes = void 0;
const logger_1 = require("@surgio/logger");
const lodash_1 = __importDefault(require("lodash"));
const constant_1 = require("../constant");
const types_1 = require("../types");
const filters_1 = require("../filters");
const index_1 = require("./index");
const logger = (0, logger_1.createLogger)({ service: 'surgio:utils:clash' });
const getClashNodes = function (list, filter) {
    return (0, filters_1.applyFilter)(list, filter)
        .map(nodeListMapper)
        .filter((item) => (0, index_1.checkNotNullish)(item));
};
exports.getClashNodes = getClashNodes;
const getClashNodeNames = function (list, filter, prependNodeNames, defaultNodeNames) {
    // istanbul ignore next
    if (arguments.length === 2 && typeof filter === 'undefined') {
        throw new Error(constant_1.ERR_INVALID_FILTER);
    }
    let result = [];
    if (prependNodeNames) {
        result = result.concat(prependNodeNames);
    }
    result = result.concat((0, exports.getClashNodes)(list, filter).map((item) => item.name));
    if (result.length === 0 && defaultNodeNames) {
        result = result.concat(defaultNodeNames);
    }
    return result;
};
exports.getClashNodeNames = getClashNodeNames;
function nodeListMapper(nodeConfig) {
    switch (nodeConfig.type) {
        case types_1.NodeTypeEnum.Shadowsocks:
            // Istanbul ignore next
            if (nodeConfig.shadowTls && !nodeConfig.clashConfig?.enableShadowTls) {
                logger.warn(`尚未开启 Clash 的 shadow-tls 支持，节点 ${nodeConfig.nodeName} 将被忽略。如需开启，请在配置文件中设置 clashConfig.enableShadowTls 为 true。`);
                return null;
            }
            // Istanbul ignore next
            if (nodeConfig.shadowTls && nodeConfig.obfs) {
                logger.warn(`Clash 不支持同时开启 shadow-tls 和 obfs，节点 ${nodeConfig.nodeName} 将被忽略。`);
                return null;
            }
            return {
                type: 'ss',
                cipher: nodeConfig.method,
                name: nodeConfig.nodeName,
                password: nodeConfig.password,
                port: nodeConfig.port,
                server: nodeConfig.hostname,
                udp: nodeConfig.udpRelay === true,
                ...(nodeConfig.obfs && ['tls', 'http'].includes(nodeConfig.obfs)
                    ? {
                        plugin: 'obfs',
                        'plugin-opts': {
                            mode: nodeConfig.obfs,
                            host: nodeConfig.obfsHost,
                        },
                    }
                    : null),
                ...(nodeConfig.obfs && ['ws', 'wss'].includes(nodeConfig.obfs)
                    ? {
                        plugin: 'v2ray-plugin',
                        'plugin-opts': {
                            mode: 'websocket',
                            tls: nodeConfig.obfs === 'wss',
                            ...(typeof nodeConfig.skipCertVerify === 'boolean' &&
                                nodeConfig.obfs === 'wss'
                                ? {
                                    'skip-cert-verify': nodeConfig.skipCertVerify,
                                }
                                : null),
                            host: nodeConfig.obfsHost,
                            path: nodeConfig.obfsUri || '/',
                            mux: typeof nodeConfig.mux === 'boolean' ? nodeConfig.mux : false,
                            headers: lodash_1.default.omit(nodeConfig.wsHeaders || {}, ['host']),
                        },
                    }
                    : null),
                ...(nodeConfig.shadowTls && nodeConfig.clashConfig?.enableShadowTls
                    ? {
                        plugin: 'shadow-tls',
                        'client-fingerprint': 'chrome',
                        'plugin-opts': {
                            password: nodeConfig.shadowTls.password,
                            ...(nodeConfig.shadowTls.version
                                ? { version: nodeConfig.shadowTls.version }
                                : null),
                            ...(nodeConfig.shadowTls.sni
                                ? { host: nodeConfig.shadowTls.sni }
                                : null),
                        },
                    }
                    : null),
            };
        case types_1.NodeTypeEnum.Vmess:
            return {
                type: 'vmess',
                cipher: nodeConfig.method,
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                udp: nodeConfig.udpRelay === true,
                uuid: nodeConfig.uuid,
                alterId: nodeConfig.alterId,
                ...(nodeConfig.network === 'tcp'
                    ? null
                    : {
                        network: nodeConfig.network,
                    }),
                tls: nodeConfig.tls,
                ...(typeof nodeConfig.skipCertVerify === 'boolean' && nodeConfig.tls
                    ? {
                        'skip-cert-verify': nodeConfig.skipCertVerify,
                    }
                    : null),
                ...(nodeConfig.network === 'ws'
                    ? {
                        'ws-opts': {
                            path: nodeConfig.path,
                            headers: {
                                ...(nodeConfig.host ? { host: nodeConfig.host } : null),
                                ...lodash_1.default.omit(nodeConfig.wsHeaders, ['host']),
                            },
                        },
                    }
                    : null),
            };
        case types_1.NodeTypeEnum.Shadowsocksr: {
            return {
                type: 'ssr',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                password: nodeConfig.password,
                obfs: nodeConfig.obfs,
                protocol: nodeConfig.protocol,
                cipher: nodeConfig.method,
                'obfs-param': nodeConfig.obfsparam ?? '',
                'protocol-param': nodeConfig.protoparam ?? '',
                udp: nodeConfig.udpRelay === true,
            };
        }
        case types_1.NodeTypeEnum.Snell:
            // Istanbul ignore next
            if (Number(nodeConfig.version) >= 4) {
                logger.warn(`Clash 尚不支持 Snell v${nodeConfig.version}，节点 ${nodeConfig.nodeName} 会被省略。`);
                return null;
            }
            return {
                type: 'snell',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                psk: nodeConfig.psk,
                'obfs-opts': {
                    mode: nodeConfig.obfs,
                    ...(nodeConfig.obfsHost
                        ? {
                            host: nodeConfig.obfsHost,
                        }
                        : null),
                },
                ...(nodeConfig.version
                    ? {
                        version: nodeConfig.version,
                    }
                    : null),
            };
        case types_1.NodeTypeEnum.HTTPS:
            return {
                type: 'http',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                username: nodeConfig.username /* istanbul ignore next */ || '',
                password: nodeConfig.password /* istanbul ignore next */ || '',
                tls: true,
                'skip-cert-verify': nodeConfig.skipCertVerify === true,
            };
        case types_1.NodeTypeEnum.HTTP:
            return {
                type: 'http',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                username: nodeConfig.username /* istanbul ignore next */ || '',
                password: nodeConfig.password /* istanbul ignore next */ || '',
            };
        case types_1.NodeTypeEnum.Trojan:
            return {
                type: 'trojan',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                password: nodeConfig.password,
                ...(nodeConfig.udpRelay ? { udp: nodeConfig.udpRelay } : null),
                ...(nodeConfig.alpn ? { alpn: nodeConfig.alpn } : null),
                ...(nodeConfig.sni ? { sni: nodeConfig.sni } : null),
                'skip-cert-verify': nodeConfig.skipCertVerify === true,
                ...(nodeConfig.network === 'ws'
                    ? {
                        network: 'ws',
                        'ws-opts': {
                            path: nodeConfig.wsPath || '/',
                            ...nodeConfig.wsHeaders,
                        },
                    }
                    : null),
            };
        case types_1.NodeTypeEnum.Socks5:
            return {
                type: 'socks5',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                ...(nodeConfig.username ? { username: nodeConfig.username } : null),
                ...(nodeConfig.password ? { password: nodeConfig.password } : null),
                ...(typeof nodeConfig.tls === 'boolean'
                    ? { tls: nodeConfig.tls }
                    : null),
                ...(typeof nodeConfig.skipCertVerify === 'boolean'
                    ? { 'skip-cert-verify': nodeConfig.skipCertVerify }
                    : null),
                ...(typeof nodeConfig.udpRelay === 'boolean'
                    ? { udp: nodeConfig.udpRelay }
                    : null),
            };
        case types_1.NodeTypeEnum.Tuic:
            // Istanbul ignore next
            if (!nodeConfig.clashConfig?.enableTuic) {
                logger.warn(`尚未开启 Clash 的 Tuic 支持，节点 ${nodeConfig.nodeName} 会被省略。如需开启，请在配置文件中设置 clashConfig.enableTuic 为 true。`);
                return null;
            }
            // Istanbul ignore next
            if (nodeConfig.alpn && !nodeConfig.alpn.length) {
                logger.warn(`节点 ${nodeConfig.nodeName} 的 alpn 为空。Stash 客户端不支持 ALPN 为空，默认的 ALPN 为 h3。`);
            }
            if ('version' in nodeConfig && Number(nodeConfig.version) >= 5) {
                return {
                    type: 'tuic',
                    name: nodeConfig.nodeName,
                    server: nodeConfig.hostname,
                    port: nodeConfig.port,
                    udp: true,
                    ...(0, index_1.pickAndFormatKeys)(nodeConfig, ['password', 'uuid', 'sni', 'skipCertVerify', 'version'], {
                        keyFormat: 'kebabCase',
                    }),
                    ...(nodeConfig.alpn ? { alpn: nodeConfig.alpn } : null),
                };
            }
            return {
                type: 'tuic',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                udp: true,
                ...(0, index_1.pickAndFormatKeys)(nodeConfig, ['token', 'sni', 'skipCertVerify', 'version'], {
                    keyFormat: 'kebabCase',
                }),
                ...(nodeConfig.alpn ? { alpn: nodeConfig.alpn } : null),
            };
        case types_1.NodeTypeEnum.Hysteria2:
            // Istanbul ignore next
            if (!nodeConfig.clashConfig?.enableHysteria2) {
                logger.warn(`尚未开启 Clash 的 Hysteria2 支持，节点 ${nodeConfig.nodeName} 会被省略。如需开启，请在配置文件中设置 clashConfig.enableHysteria2 为 true。`);
                return null;
            }
            return {
                type: 'hysteria2',
                name: nodeConfig.nodeName,
                server: nodeConfig.hostname,
                port: nodeConfig.port,
                [nodeConfig.clashConfig.clashCore === 'stash' ? 'auth' : 'password']: nodeConfig.password,
                up: nodeConfig.uploadBandwidth || 0,
                down: nodeConfig.downloadBandwidth || 0,
                ...(0, index_1.pickAndFormatKeys)(nodeConfig, ['obfs', 'obfsPassword', 'sni', 'skipCertVerify'], {
                    keyFormat: 'kebabCase',
                }),
                ...(nodeConfig.alpn ? { alpn: nodeConfig.alpn } : null),
            };
        case types_1.NodeTypeEnum.Wireguard:
            // istanbul ignore next
            if (nodeConfig.peers.length > 1) {
                logger.warn(`节点 ${nodeConfig.nodeName} 有多个 WireGuard Peer，然而 Stash 或 Clash 仅支持一个 Peer，因此只会使用第一个 Peer。`);
            }
            return {
                type: 'wireguard',
                name: nodeConfig.nodeName,
                'private-key': nodeConfig.privateKey,
                ip: nodeConfig.selfIp,
                ...(nodeConfig.selfIpV6 ? { ipv6: nodeConfig.selfIpV6 } : null),
                ...(nodeConfig.mtu ? { mtu: nodeConfig.mtu } : null),
                ...(nodeConfig.dnsServers ? { dns: nodeConfig.dnsServers } : null),
                udp: true,
                // Peer
                server: (0, index_1.getHostnameFromHost)(nodeConfig.peers[0].endpoint),
                port: (0, index_1.getPortFromHost)(nodeConfig.peers[0].endpoint),
                'public-key': nodeConfig.peers[0].publicKey,
                ...(nodeConfig.peers[0].presharedKey
                    ? { 'preshared-key': nodeConfig.peers[0].presharedKey }
                    : null),
                ...(nodeConfig.peers[0].reservedBits
                    ? {
                        reserved: nodeConfig.peers[0].reservedBits,
                    }
                    : null),
            };
        // istanbul ignore next
        default:
            logger.warn(`不支持为 Clash 生成 ${nodeConfig.type} 的节点，节点 ${nodeConfig.nodeName} 会被省略`);
            return null;
    }
}
//# sourceMappingURL=clash.js.map